'' =================================================================================================
''
''   original File....... jm_freqin.spin
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''
'' =================================================================================================

{{

  This object uses ctra and ctrb of its own cog to measure the period of an intput waveform.
  The period is measured in clock ticks; this value can be divided into the Propeller clock
  frequecy to determine the frequency of the input waveform.  In application the period is
  divided into 10x the clock frequency to increase the resolution to 0.1Hz; this is espeically
  helpful for low frequencies.  Estimated range is 0.5Hz to ~40MHz (using 80MHz clkfreq).

  The counters are setup such that ctra measures the high phase of the input and ctrb measures
  low phase.  Measuring each phase independently allows the input waveform to be asymmetric.

  In order to prevent a loss of signal from causing an eroneous value from the .freq() method
  the fcCyles value is cleared after a valid frequency is calculated; this means that you
  should not call this method at a rate faster than the expected input frequency.  

}}


var

  long  cog

  long  fcPin                                                   ' frequency counter pin
  long  fcCycles                                                ' frequency counter cycles
  long  fcToothcount
  long  fcAverage
  long  fcMax
  long  fcMin
  long  fcToothCntMax
  long  fcGapCnt
  long  fcGapCntPeak 

pub init(p) : okay

'' Start frequency counter on pin p
'' -- valid input pins are 0..27

  if p < 28                                                     ' protect rx, tx, i2c
    fcPin := p
    fcCycles := 0
    okay := cog := cognew(@frcntr, @fcPin) + 1
  else
    okay := false


pub cleanup

'' Stop frequency counter cog if running

  if cog
    cogstop(cog~ - 1)

pub maxCyc
  result := fcMax
  
pub minCyc
  result := fcMin
  
pub average
  result := fcAverage
  
pub toothcnt
  result := fcToothcount

pub toothCntMax
  result := fcToothCntMax

pub gapCnt
  result := fcGapCnt  
  
pub period

'' Returns period of input waveform

  'return fcCycles
  result := fcCycles
  'fcCycles~ 
  

pub freq | p, f

'' Converts input period to frequency
'' -- returns frequency in 0.1Hz units (1Hz = 10 units)
'' -- should not be called faster than expected minimum input frequency

  p := period
  if p
    f := clkfreq * 10 / p                                       ' calculate frequency
    'fcCycles := 0                                               ' clear for loss of input
  else
    f := 0

  return f
    

dat

                        org     0

frcntr                  mov     tmp1, par                       ' * start of structure, passes an address to the value par represents and copies that address to tmp1
                        rdlong  tmp2, tmp1                      ' * get beginning variable address location, copy into tmp2

                        mov     ctra, POS_DETECT                ' * ctra measures high phase
                        add     ctra, tmp2                      ' * add tmp2 to ctra and write to ctra
                        mov     frqa, #1                        ' * move frqa 1 byte?
                        
                        mov     ctrb, NEG_DETECT                ' * ctrb measures low phase
                        add     ctrb, tmp2                      ' * add tmp2 to ctrb and write to ctrb
                        mov     frqb, #1                        ' * move frqb 1 byte? 
                        
                        mov     mask, #1                        ' * create pin mask
                        shl     mask, tmp2                      ' * shift left the mask, number of bytes of tmp2? 

                        ' save addresses of hub storage
                        add     cyclepntr, tmp1                  ' * adds value of cyclpntr and tmp1 and writes that to cyclepntr
                        add     toothcntptr, tmp1                ' * adds value of toothcntptr and tmp1 and writes that to cyclepntr
                        add     averageptr, tmp1                 ' * adds value of averageptr and tmp1 and writes that to cyclepntr
                        add     maxptr, tmp1                     ' * adds value of maxptr and tmp1 and writes that to cyclepntr
                        add     minptr, tmp1                     ' * adds value of minptr and tmp1 and writes that to cyclepntr
                        add     toothCntMaxptr, tmp1             ' * adds value of minptr and tmp1 and writes that to cyclepntr
                        add     gapCntAsmptr, tmp1
                        add     gapCntAsmPeakptr, tmp1

                        'mov toothCntMaxAsm, #0  ' * initializes toothCntMaxAsm to 0 ?
                        

' loop starts here
                       
restart                 waitpne mask, mask                      ' * wait for 0 phase
                        mov     cycles, phsa                    ' * copies phsA to cycles
                        mov     phsa, #0                        ' * clear high phase counter

                        ' wait for pin == 1
highphase               waitpeq mask, mask                      ' * wait for pin to equal 1
                        add     cycles, phsb                    ' * adds phsB to cycles (already containing phsA)
                        mov     phsb, #0                        ' * clear low phase counter

                        ' write number of cycles measured to fcCycles
                        wrlong  cycles, cyclepntr               ' * writes cycles to cyclepntr which is really fcCycles address

                        ' count the teeth
                        cmp     tooth, #36 WZ,WC' *compare unsigned values of tooth and literal number 36, some sort of post fix flags?
if_ae                   mov     tooth, #1       ' *if above or equal, move to tooth
if_b                    add     tooth, #1       ' *if below, add 1 to tooth

                        ' write number of actual tooth to fcToothcount
                        wrlong  tooth, toothcntptr ' * write the value of tooth to the toothcntptr
                        
                        cmp     toothCntMaxAsm, tooth  WZ, WC ' Z should be 0, C should be 1 
if_NZ_AND_C             mov     toothCntMaxAsm, tooth          ' * copy tooth to toothCntMaxAsm
if_NZ_AND_C             wrlong  toothCntMaxAsm, toothCntMaxptr ' * write the value of toothCntMaxAsm to the toothCntMaxptr                        

                        ' keep track of the maximum and write result to fcMax
                        cmp     maxc, cycles WZ,WC ' * write flags?
if_b                    mov     maxc, cycles    ' *if below
                        wrlong  maxc, maxptr    ' * write the value of maxc to the maxptr          
                        ' keep track of the minimum and write result to fcMin
                        cmp     minc, cycles WZ,WC  ' * write flags?
if_a                    mov     minc, cycles    ' *if above
                        wrlong  minc, minptr    ' * write the value of minc to the minptr

                        ' gap-detect! If gap detected don't add this extra long
                        ' cycle to the average
                        cmp     dontcount,cycles WZ,WC,NR ' * write flags?
if_b                    add     gapCntAsm, #1
if_b                    wrlong  gapCntAsm, gapCntAsmptr
                        ' * if timeStamp + clkfreq => cnt
{                        cmp     time + clkfreq, cnt WZ,WC
if_b                    mov     gapCntAsmPeak, gapCntAsm
if_b                    wrlong  gapCntAsmPeak, gapCntAsmPeakptr   }                   

                        cmp     dontcount,cycles WZ,WC,NR
if_b                    jmp     #restart        ' *if below goto restart

                        ' moving-average (remove one average value and add the current)
                        sub     database, averagex  ' * subtract averagex from database (unsigned)
                        add     database, cycles ' * add cycles to database and write value to database

                        ' this code is for waiting 8 cycles before start of calculation
                        ' of the average
calcavg                 djnz    avgcnt, #restart
                        ' this will replace the previous conditional jump with a NOP
                        ' after the 8th cycle, so that all following cycles will directly
                        ' go to the calculation below
                        mov     calcavg, nopinstr ' * move calcavg to nopinstr

                        ' divide the sum of cycles by 8 and write result to fcAverage
                        mov     averagex, database ' * move averagex to database
                        shr     averagex, #3  ' * shift bits right by 3
                        wrlong  averagex, averageptr ' * write averagex to averageptr

                        ' do the next loop
                        jmp     #restart ' * go to restart

                        ' this is the NOP which is copied to calcavg
nopinstr                mov     0, 0 NR  ' * some sort of post fix flag?                      
' --------------------------------------------------------------------------------------------------

POS_DETECT              long    %01000 << 26 
NEG_DETECT              long    %01100 << 26

cyclepntr               long    4
toothcntptr             long    8
averageptr              long    12
maxptr                  long    16
minptr                  long    20
toothCntMaxPtr          long    24
gapCntAsmptr            long    28
gapCntAsmPeakptr        long    32  

maxc                    long    0
minc                    long    -1

tooth                   long    0
toothCntMaxAsm          long    0
gapCntAsm               long    0
gapCntAsmPeak           long    0

database                long    0
averagex                
avgcnt                  long    8
dontcount               long    13400


tmp1                    res     1
tmp2                    res     1

mask                    res     1                               ' mask for frequency input pin
cycles                  res     1                               ' cycles in input period


                        fit     492
                        

dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   